/*
hmc5883l lib sample

copyright (c) Davide Gironi, 2012

Released under GPLv3.
Please refer to LICENSE file for licensing information.
*/


#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "math.h"

#include "hmc5883l.h"

#define MAGN_DOCALIBRATION 0 //enable or disable magnetometer calibration
#define magncal_getrawdata(mxraw, myraw, mzraw) hmc5883l_getrawdata(mxraw, myraw, mzraw); //set the function that return magnetometer raw values
#include "magn_docalibration.h"

#define UART_BAUD_RATE 57600
#include "uart.h"


int steps = 0;
int direction = 1;
int steps_left = 0;
int steps_per_degree = 12;
int prev_heading = 0;
int heading = 0;

int randomDegree()
{
	return rand() % 360;
}

void uln2003_setup()
{
	/* Setez pinii PB0-3 ca iesire. */
	DDRB |= (1 << PB0);
	DDRB |= (1 << PB1);
	DDRB |= (1 << PB2);
	DDRB |= (1 << PB3);
}

void updateStep()
{
	if(direction == 1)
		steps++;
	if(direction == 0)
		steps--;

	if(steps > 7)
		steps = 0;
	if(steps < 0)
		steps = 7;
}

void stepper()
{	
	switch(steps) {
	case 0:
		PORTB &= ~(1 << PB0);
		PORTB &= ~(1 << PB1);
		PORTB &= ~(1 << PB2);
		PORTB |= (1 << PB3);
		break;

	case 1:
		PORTB &= ~(1 << PB0);
		PORTB &= ~(1 << PB1);
		PORTB |= (1 << PB2);
		PORTB |= (1 << PB3);
		break;

	case 2:
		PORTB &= ~(1 << PB0);
		PORTB &= ~(1 << PB1);
		PORTB |= (1 << PB2);
		PORTB &= ~(1 << PB3);
		break;

	case 3:
		PORTB &= ~(1 << PB0);
		PORTB |= (1 << PB1);
		PORTB |= (1 << PB2);
		PORTB &= ~(1 << PB3);
		break;

	case 4:
		PORTB &= ~(1 << PB0);
		PORTB |= (1 << PB1);
		PORTB &= ~(1 << PB2);
		PORTB &= ~(1 << PB3);
		break;

	case 5:
		PORTB |= (1 << PB0);
		PORTB |= (1 << PB1);
		PORTB &= ~(1 << PB2);
		PORTB &= ~(1 << PB3);
		break;

	case 6:
		PORTB |= (1 << PB0);
		PORTB &= ~(1 << PB1);
		PORTB &= ~(1 << PB2);
		PORTB &= ~(1 << PB3);
		break;

	case 7:
		PORTB |= (1 << PB0);
		PORTB &= ~(1 << PB1);
		PORTB &= ~(1 << PB2);
		PORTB |= (1 << PB3);
		break;

	default:
		PORTB &= ~(1 << PB0);
		PORTB &= ~(1 << PB1);
		PORTB &= ~(1 << PB2);
		PORTB &= ~(1 << PB3);
		break;
	}

	updateStep();
}

void loop()
{
	while (steps_left > 0) {
		stepper(); 
		steps_left--;
		_delay_ms(1);
	}
}


int main(void) {
    int16_t mxraw = 0;
    int16_t myraw = 0;
    int16_t mzraw = 0;
    double mx = 0;
    double my = 0;
    double mz = 0;
	int delta_heading;

	//char itmp[10];

    //init uart
	//uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));

	//init hmc5883l
	hmc5883l_init();

	#if MAGN_DOCALIBRATION  == 1
	magncal_docalibrationclient(uart_putc, uart_getc);
	#endif
	
	uln2003_setup();

	/* Setez pinul PD7 ca iesire. */
	DDRD |= (1 << PD7); 
	
	/* Sting LED-ul. */
	PORTD &= ~(1 << PD7);
		

	while(1) {
		hmc5883l_getrawdata(&mxraw, &myraw, &mzraw);
		hmc5883l_getdata(&mx, &my, &mz);

		//get magnetic heading (in degrees)
		heading = (int) (atan2((double)myraw,(double)mxraw)*57.29578);
		
		//add magnetic declination (optional)
		//get magnetic declination (in degrees) http://magnetic-declination.com
		//if you have an EAST declination use +, if you have a WEST declination use -
		//es, my declination is 1.73 positive
		//float declination = 1.73;
		//heading += declination;
		//check 360degree heading
		
		if(heading < 0)
			heading = 360 + heading;

		/*itoa(mxraw, itmp, 10); uart_puts(itmp); uart_putc(' ');
		itoa(myraw, itmp, 10); uart_puts(itmp); uart_putc(' ');
		itoa(mzraw, itmp, 10); uart_puts(itmp); uart_putc(' ');
		dtostrf(mx, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		dtostrf(my, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		dtostrf(mz, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		dtostrf(heading, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		uart_puts("\r\n");*/
		
		//heading = randomDegree();
		delta_heading = heading - prev_heading;
		prev_heading = heading;
		
		
		if (abs(delta_heading) > 180) {
			if (delta_heading < 0)
				delta_heading = 360 + delta_heading;
			else if (delta_heading > 0)
				delta_heading = delta_heading - 360;
		}
		
		/* rotate right */
		if (delta_heading > 0) {
			direction = 1;
		}
		/* rotate left */
		else if (delta_heading < 0){
			direction = 0;
		}
		
		steps_left = steps_per_degree * abs(delta_heading);
		steps = 0;
		
		/* Aprind LED-ul. */
		PORTD |= (1 << PD7);
		
		loop();
		
		/* Sting LED-ul. */
		PORTD &= ~(1 << PD7);
		
		_delay_ms(500);
	}
}
